module Data.Serialize.Ldsf.Lean.Category.Browser.V0 (
      category
    , version
    , functionOpen
    , functionEmbed
    , lookupTable
    ) where

import Data.Map (Map)
import qualified Data.Map as Map
import Data.Serialize.Ldsf
import qualified Data.Serialize.Ldsf as Ldsf
import Data.Serialize.Ldsf.Types
import Data.Serialize.Ldsf.Lean.InterfaceFunction (InterfaceFunction(InterfaceFunction))
import qualified Data.Serialize.Ldsf.Lean.InterfaceFunction as IF
import qualified Data.Text as T
import Network.Lean.Types

ldsfVersion :: Ldsf.Version
ldsfVersion = 0

category :: InterfaceCategory
category = Browser

version :: InterfaceVersion
version = 0

functionOpen :: InterfaceFunction
functionOpen = InterfaceFunction
    { IF.id = 0
    , IF.ldsfVersion = ldsfVersion
    , IF.category = category
    , IF.version = version
    , IF.name = T.pack "open"
    , IF.getArgs = getArgs
    , IF.putArgs = putArgs
    , IF.getResult = getResult
    , IF.putResult = putResult
    }
    where
        getArgs = getText16 >>= \v -> pure [ValueText16 v]
        putArgs [ValueText16 v] = [ValueText16 v]
        putArgs _              = fail "Expected type ValueText16" -- TODO: is fail good!?!?
        getResult = pure [ValueNothing]
        putResult [ValueNothing] = [ValueNothing]
        putResult _              = fail "Expected type ValueNothing" -- TODO: is fail good!?!?

functionEmbed :: InterfaceFunction
functionEmbed = InterfaceFunction
    { IF.id = 1
    , IF.ldsfVersion = ldsfVersion
    , IF.category = category
    , IF.version = version
    , IF.name = T.pack "embed"
    , IF.getArgs = getArgs
    , IF.putArgs = putArgs
    , IF.getResult = getResult
    , IF.putResult = putResult
    }
    where
        getArgs = do
            state <- getUInt8
            windowId <- getUInt32
            pure [ValueUInt8 state, ValueUInt32 windowId]
        putArgs [ValueUInt8 x, ValueUInt32 y] = [ValueUInt8 x, ValueUInt32 y]
        putArgs _                             = fail "Expected type ValueUInt8 followed by ValueUInt32" -- TODO: is fail good!?!?
        getResult = pure [ValueNothing]
        putResult [ValueNothing] = [ValueNothing]
        putResult _              = fail "Expected type ValueNothing" -- TODO: is fail good!?!?

lookupTable :: Map (Ldsf.Version, InterfaceCategory, InterfaceVersion, InterfaceFunctionId) InterfaceFunction
lookupTable = makeLookupTable
    [ functionOpen
    , functionEmbed
    ]

makeLookupEntry :: InterfaceFunction -> ((Ldsf.Version, InterfaceCategory, InterfaceVersion, InterfaceFunctionId), InterfaceFunction)
makeLookupEntry x = ((IF.ldsfVersion x, IF.category x, IF.version x, IF.id x), x)

makeLookupTable :: [InterfaceFunction] -> Map (Ldsf.Version, InterfaceCategory, InterfaceVersion, InterfaceFunctionId) InterfaceFunction
makeLookupTable xs = Map.fromList $ map makeLookupEntry xs
